@@ -31,16 +31,22 @@ const NO_FLAG_SETTINGS: &[AppSettings] = &[
 
 #[derive(StructOpt, Debug, Serialize)]
 pub struct RootArgs {
-    #[structopt(long, hidden = true)]
-    pub dump_args: bool,
-    #[structopt(long, global = true, help = "Do not record history")]
-    pub no_trace: bool,
-    #[structopt(short = "A", long, global = true, help = "Show scripts NOT within recent days", conflicts_with_all = &["all", "timeless"])]
-    pub archaeology: bool,
-    #[structopt(long, global = true)]
-    pub no_alias: bool, // NOTE: no-alias 的判斷其實存在於 structopt 之外，寫在這裡只是為了生成幫助訊息
     #[structopt(short = "H", long, help = "Path to hyper script home")]
     pub hs_home: Option<String>,
+    #[structopt(long, hidden = true)]
+    pub dump_args: bool,
+    #[structopt(long, help = "Do not record history")]
+    pub no_trace: bool,
+    #[structopt(
+        long,
+        conflicts_with = "no-trace",
+        help = "Do not affect script time (but will still record history)"
+    )]
+    pub humble: bool,
+    #[structopt(short = "A", long, global = true, help = "Show scripts NOT within recent days", conflicts_with_all = &["all", "timeless"])]
+    pub archaeology: bool,
+    #[structopt(long)]
+    pub no_alias: bool, // NOTE: no-alias 的判斷其實存在於 structopt 之外，寫在這裡只是為了生成幫助訊息
     #[structopt(
         short,
         long,
@@ -159,6 +165,8 @@ pub enum Subs {
     },
     #[structopt(setting = AppSettings::Hidden)]
     LoadUtils,
+    #[structopt(about = "Migrate the database")]
+    Migrate,
     #[structopt(about = "Edit hyper script", settings = &[AllowLeadingHyphen, TrailingVarArg])]
     Edit {
         #[structopt(
@@ -180,6 +188,8 @@ pub enum Subs {
     },
     #[structopt(about = "Manage alias", settings = NO_FLAG_SETTINGS)]
     Alias {
+        #[structopt(long, conflicts_with_all = &["before", "after"])]
+        short: bool,
         #[structopt(
             long,
             short,
@@ -274,9 +284,11 @@ pub enum History {
     // TODO: 好想把它寫在 history rm 裡面...
     #[structopt(
         name = "rm-id",
-        about = "Remove history by the event's id\nUseful if you want to keep those illegal arguments from polluting the history."
+        about = "Remove an event by it's id.\nUseful if you want to keep those illegal arguments from polluting the history."
     )]
-    RMID { event_id: u64 }, // TODO: 測試
+    RMID { event_id: u64 },
+    #[structopt(about = "Humble an event by it's id")]
+    Humble { event_id: u64 },
     Show {
         #[structopt(default_value = "-", parse(try_from_str))]
         script: ScriptQuery,
@@ -293,7 +305,7 @@ pub enum History {
         #[structopt(parse(try_from_str), required = true, min_values = 1)]
         queries: Vec<ListQuery>,
     },
-    #[structopt( settings = NO_FLAG_SETTINGS)] // TODO: 測試
+    #[structopt( settings = NO_FLAG_SETTINGS)]
     Amend {
         event_id: u64,
         #[structopt(help = "Command line args to pass to the script")]
@@ -451,10 +463,12 @@ mod test {
         match &args.subcmd {
             Some(Subs::Alias {
                 unset,
-                before: Some(before),
+                short,
                 after,
+                before: Some(before),
             }) => {
                 assert_eq!(*unset, false);
+                assert_eq!(*short, false);
                 assert_eq!(before, "trash");
                 assert_eq!(after, &["-f", "removed"]);
             }
