@@ -6,7 +6,7 @@ use hyper_scripter::extract_msg::{extract_env_from_content, extract_help_from_co
 use hyper_scripter::list::{fmt_list, DisplayIdentStyle, DisplayStyle, ListOptions};
 use hyper_scripter::path;
 use hyper_scripter::query::{self, RangeQuery, ScriptQuery};
-use hyper_scripter::script_repo::RepoEntry;
+use hyper_scripter::script_repo::{RepoEntry, ScriptRepo};
 use hyper_scripter::script_time::ScriptTime;
 use hyper_scripter::tag::{Tag, TagFilter};
 use hyper_scripter::util::{
@@ -15,6 +15,8 @@ use hyper_scripter::util::{
     print_iter,
 };
 use hyper_scripter::Either;
+use hyper_scripter::{db, migration};
+use hyper_scripter_historian::Historian;
 
 #[tokio::main]
 async fn main() {
@@ -54,6 +56,14 @@ async fn main_err_handle() -> Result<Vec<Error>> {
         return Ok(vec![]);
     }
 
+    root.set_home_unless_from_alias()?;
+
+    if matches!(root.subcmd, Some(Subs::Migrate)) {
+        migration::do_migrate(db::get_file()).await?;
+        Historian::do_migrate(path::get_home()).await?;
+        return Ok(vec![]);
+    }
+
     let res = main_inner(root).await?;
     if let Some(conf) = res.conf {
         log::info!("存入改變後的設定檔");
@@ -72,7 +82,6 @@ struct MainReturn {
 }
 
 async fn main_inner(root: Root) -> Result<MainReturn> {
-    root.set_home_unless_from_alias()?;
     Config::set_prompt_level(root.root_args.prompt_level);
     let explicit_filter = !root.root_args.filter.is_empty();
 
@@ -99,6 +108,7 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
             unset: false,
             before: Some(before),
             after,
+            ..
         } => {
             if !after.is_empty() {
                 log::info!("設定別名 {} {:?}", before, after);
@@ -130,12 +140,17 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
         Subs::Alias {
             unset: false,
             before: None,
+            short,
             ..
         } => {
             log::info!("印出所有別名");
             for (before, alias) in conf.alias.iter() {
-                let after = alias.after.join(" ");
-                println!("{}=\"{}\"", before, after);
+                print!("{}", before);
+                if !short {
+                    let after = alias.after.join(" ");
+                    print!("=\"{}\"", after);
+                }
+                println!("");
             }
         }
         Subs::Edit {
@@ -390,26 +405,13 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
         Subs::Tags(Tags {
             subcmd: Some(TagsSubs::LS { named, known }),
         }) => {
-            let mut known_tags_iter = || {
-                let mut map: HashMap<&Tag, u32> = Default::default();
-                for script in repo.iter_mut(true) {
-                    let script = script.into_inner();
-                    for tag in script.tags.iter() {
-                        let entry = map.entry(tag).or_insert(0);
-                        *entry += 1;
-                    }
-                }
-                let mut v: Vec<_> = map.into_iter().map(|(k, v)| (k.clone(), v)).collect();
-                v.sort_by_key(|(_, v)| std::cmp::Reverse(*v));
-                v.into_iter().map(|(k, _)| k)
-            };
             if named {
                 print_iter(conf.tag_filters.iter().map(|f| &f.name), " ");
             } else if known {
-                print_iter(known_tags_iter(), " ");
+                print_iter(known_tags_iter(&mut repo), " ");
             } else {
                 print!("known tags:\n  ");
-                print_iter(known_tags_iter(), " ");
+                print_iter(known_tags_iter(&mut repo), " ");
                 println!("");
                 println!("tag filters:");
                 for filter in conf.tag_filters.iter() {
@@ -476,19 +478,12 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
         Subs::History {
             subcmd: History::RMID { event_id },
         } => {
-            let res = historian.ignore_args_by_id(event_id as i64).await?;
-            if let Some(res) = res {
-                let mut entry = repo.get_mut_by_id(res.script_id).ok_or_else(|| {
-                    log::error!("史學家給的腳本 id 竟然在倉庫中找不到……");
-                    Error::ScriptNotFound(res.script_id.to_string())
-                })?;
-                entry
-                    .update(|info| {
-                        info.exec_time = res.exec_time.map(|t| ScriptTime::new(t));
-                        info.exec_done_time = res.exec_done_time.map(|t| ScriptTime::new(t));
-                    })
-                    .await?;
-            }
+            process_event_by_id(false, &mut repo, &historian, event_id).await?;
+        }
+        Subs::History {
+            subcmd: History::Humble { event_id },
+        } => {
+            process_event_by_id(true, &mut repo, &historian, event_id).await?;
         }
         Subs::History {
             subcmd: History::Amend { event_id, args },
@@ -547,3 +542,54 @@ async fn main_inner(root: Root) -> Result<MainReturn> {
 async fn create_read_event(entry: &mut RepoEntry<'_>) -> Result<i64> {
     entry.update(|info| info.read()).await
 }
+
+fn known_tags_iter<'a>(repo: &'a mut ScriptRepo) -> impl Iterator<Item = &'a Tag> {
+    use std::collections::hash_map::Entry::*;
+
+    let mut map: HashMap<&Tag, _> = Default::default();
+    for script in repo.iter_mut(true) {
+        let script = script.into_inner();
+        let date = script.last_major_time();
+        for tag in script.tags.iter() {
+            match map.entry(tag) {
+                Occupied(entry) => {
+                    let date_mut = entry.into_mut();
+                    *date_mut = std::cmp::max(date, *date_mut);
+                }
+                Vacant(entry) => {
+                    entry.insert(date);
+                }
+            }
+        }
+    }
+    let mut v: Vec<_> = map.into_iter().map(|(k, v)| (k, v)).collect();
+    v.sort_by_key(|(_, v)| std::cmp::Reverse(*v));
+    v.into_iter().map(|(k, _)| k)
+}
+
+async fn process_event_by_id(
+    is_humble: bool,
+    repo: &mut ScriptRepo,
+    historian: &Historian,
+    event_id: u64,
+) -> Result {
+    let event_id = event_id as i64;
+    let res = if is_humble {
+        historian.humble_args_by_id(event_id).await?
+    } else {
+        historian.ignore_args_by_id(event_id).await?
+    };
+    if let Some(res) = res {
+        let mut entry = repo.get_mut_by_id(res.script_id).ok_or_else(|| {
+            log::error!("史學家給的腳本 id 竟然在倉庫中找不到……");
+            Error::ScriptNotFound(res.script_id.to_string())
+        })?;
+        entry
+            .update(|info| {
+                info.exec_time = res.exec_time.map(|t| ScriptTime::new(t));
+                info.exec_done_time = res.exec_done_time.map(|t| ScriptTime::new(t));
+            })
+            .await?;
+    }
+    Ok(())
+}
