@@ -125,12 +125,12 @@ impl Default for Config {
                     inactivated: false,
                 },
                 NamedTagFilter {
-                    content: "+m/all,^hide".parse().unwrap(),
+                    content: "+all,^hide!".parse().unwrap(),
                     name: "no-hidden".to_owned(),
                     inactivated: false,
                 },
                 NamedTagFilter {
-                    content: "+m/all,^removed".parse().unwrap(),
+                    content: "+all,^removed!".parse().unwrap(),
                     name: "no-removed".to_owned(),
                     inactivated: false,
                 },
