@@ -2,7 +2,7 @@ use super::{
     exec_time_str, extract_help, style, tree, DisplayIdentStyle, DisplayStyle, Grouping,
     ListOptions,
 };
-use crate::error::{Error, Result};
+use crate::error::Result;
 use crate::query::do_list_query;
 use crate::script::ScriptInfo;
 use crate::script_repo::ScriptRepo;
@@ -125,10 +125,7 @@ pub async fn fmt_list<W: Write>(
 ) -> Result<()> {
     let mut opt = convert_opt(w, opt);
 
-    let latest_script_id = script_repo
-        .latest_mut(1, false)
-        .ok_or_else(|| Error::Empty)?
-        .id;
+    let latest_script_id = script_repo.latest_mut(1, false).map_or(-1, |s| s.id);
 
     if let DisplayStyle::Long(table) = &mut opt.display_style {
         if opt.grouping != Grouping::Tree {
