@@ -18,9 +18,18 @@ pub struct RecentFilter {
     pub archaeology: bool,
 }
 
+#[derive(Debug)]
+enum TraceOption {
+    Normal,
+    // record nothing
+    NoTrace,
+    // don't affect last time, only record history
+    Humble,
+}
+
 #[derive(Debug)]
 pub struct DBEnv {
-    no_trace: bool,
+    trace_opt: TraceOption,
     info_pool: SqlitePool,
     historian: Historian,
     modifies_script: bool,
@@ -64,23 +73,30 @@ impl DBEnv {
     }
 
     async fn update_last_time(&self, info: &ScriptInfo) -> Result {
+        if !matches!(self.trace_opt, TraceOption::Normal) {
+            return Ok(());
+        }
+
         let last_time = info.last_time();
         let exec_time = info.exec_time.as_ref().map(|t| **t);
         let exec_done_time = info.exec_done_time.as_ref().map(|t| **t);
-        let neglect = info.neglect_time.as_ref().map(|t| **t);
+        let neglect_time = info.neglect_time.as_ref().map(|t| **t);
+        let miss_time = info.miss_time.as_ref().map(|t| **t);
         let exec_count = info.exec_count as i32;
         sqlx::query!(
             "
             INSERT OR REPLACE INTO last_events
-            (script_id, last_time, read, write, exec, exec_done, neglect, exec_count) VALUES(?, ?, ?, ?, ?, ?, ?, ?)
+            (script_id, last_time, read, write, miss, exec, exec_done, neglect, exec_count)
+            VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?)
             ",
             info.id,
             last_time,
             *info.read_time,
             *info.write_time,
+            miss_time,
             exec_time,
             exec_done_time,
-            neglect,
+            neglect_time,
             exec_count
         )
         .execute(&self.info_pool)
@@ -144,26 +160,33 @@ impl DBEnv {
             .await?;
         }
 
-        if self.no_trace {
+        if matches!(self.trace_opt, TraceOption::NoTrace) {
             return Ok(0);
         }
 
         let mut last_event_id = 0;
+        macro_rules! record_event {
+            ($time:expr, $data:expr) => {
+                self.historian.record(&Event {
+                    script_id: info.id,
+                    humble: matches!(self.trace_opt, TraceOption::Humble),
+                    time: $time,
+                    data: $data,
+                })
+            };
+        }
 
         if let Some(time) = info.exec_done_time.as_ref() {
             if let Some(&(code, main_event_id)) = time.data() {
                 log::debug!("{:?} 的執行完畢事件", info.name);
-                last_event_id = self
-                    .historian
-                    .record(&Event {
-                        script_id: info.id,
-                        data: EventData::ExecDone {
-                            code,
-                            main_event_id,
-                        },
-                        time: **time,
-                    })
-                    .await?;
+                last_event_id = record_event!(
+                    **time,
+                    EventData::ExecDone {
+                        code,
+                        main_event_id,
+                    }
+                )
+                .await?;
 
                 if last_event_id != 0 {
                     self.update_last_time(info).await?;
@@ -178,41 +201,30 @@ impl DBEnv {
 
         if info.read_time.has_changed() {
             log::debug!("{:?} 的讀取事件", info.name);
-            last_event_id = self
-                .historian
-                .record(&Event {
-                    script_id: info.id,
-                    data: EventData::Read,
-                    time: *info.read_time,
-                })
-                .await?;
+            last_event_id = record_event!(*info.read_time, EventData::Read).await?;
         }
         if info.write_time.has_changed() {
             log::debug!("{:?} 的寫入事件", info.name);
-            last_event_id = self
-                .historian
-                .record(&Event {
-                    script_id: info.id,
-                    data: EventData::Write,
-                    time: *info.write_time,
-                })
-                .await?;
+            last_event_id = record_event!(*info.write_time, EventData::Write).await?;
+        }
+        if let Some(time) = info.miss_time.as_ref() {
+            if time.has_changed() {
+                log::debug!("{:?} 的錯過事件", info.name);
+                last_event_id = record_event!(**time, EventData::Miss).await?;
+            }
         }
         if let Some(time) = info.exec_time.as_ref() {
             if let Some((content, args, dir)) = time.data() {
                 log::debug!("{:?} 的執行事件", info.name);
-                last_event_id = self
-                    .historian
-                    .record(&Event {
-                        script_id: info.id,
-                        data: EventData::Exec {
-                            content,
-                            args,
-                            dir: dir.as_deref(),
-                        },
-                        time: **time,
-                    })
-                    .await?;
+                last_event_id = record_event!(
+                    **time,
+                    EventData::Exec {
+                        content,
+                        args,
+                        dir: dir.as_deref(),
+                    }
+                )
+                .await?;
             }
         }
 
@@ -262,7 +274,6 @@ impl ScriptRepo {
         pool: SqlitePool,
         recent: Option<RecentFilter>,
         historian: Historian,
-        no_trace: bool,
         modifies_script: bool,
     ) -> Result<ScriptRepo> {
         let mut hidden_map = HashMap::<String, ScriptInfo>::default();
@@ -305,6 +316,9 @@ impl ScriptRepo {
             if let Some(time) = record.read {
                 builder.read_time(time);
             }
+            if let Some(time) = record.miss {
+                builder.miss_time(time);
+            }
             if let Some(time) = record.exec {
                 builder.exec_time(time);
             }
@@ -322,7 +336,7 @@ impl ScriptRepo {
                     log::debug!("腳本 {} 曾於 {} 被忽略", script.name, neglect);
                     time_bound = std::cmp::max(neglect, time_bound);
                 }
-                let overtime = time_bound > script.last_time_without_read();
+                let overtime = time_bound > script.last_major_time();
                 archaeology ^ overtime
             } else {
                 false
@@ -338,13 +352,19 @@ impl ScriptRepo {
             hidden_map,
             latest_name: None,
             db_env: DBEnv {
+                trace_opt: TraceOption::Normal,
                 info_pool: pool,
-                no_trace,
                 historian,
                 modifies_script,
             },
         })
     }
+    pub fn no_trace(&mut self) {
+        self.db_env.trace_opt = TraceOption::NoTrace;
+    }
+    pub fn humble(&mut self) {
+        self.db_env.trace_opt = TraceOption::Humble;
+    }
     // fn latest_mut_no_cache(&mut self) -> Option<&mut ScriptInfo<'a>> {
     //     let latest = self.map.iter_mut().max_by_key(|(_, info)| info.last_time());
     //     if let Some((name, info)) = latest {
