@@ -1,4 +1,4 @@
-use super::{init_repo, print_iter};
+use super::{init_repo, print_iter, to_display_args};
 use crate::args::{AliasRoot, Completion, Root, Subs};
 use crate::config::Config;
 use crate::error::{Error, Result};
@@ -93,8 +93,13 @@ pub async fn handle_completion(comp: Completion) -> Result {
             };
 
             print_iter(scripts.iter().map(|s| s.name.key()), " ");
-
-            Ok(())
+        }
+        Completion::NoSubcommand { args } => {
+            if let Ok(root) = parse_alias_root(&args) {
+                if root.subcmd.is_some() {
+                    return Err(Error::Completion);
+                }
+            } // else: 解析錯誤當然不可能有子命令啦
         }
         Completion::Alias { args } => {
             let root = parse_alias_root(&args)?;
@@ -103,15 +108,35 @@ pub async fn handle_completion(comp: Completion) -> Result {
             if let Some(new_args) = root.expand_alias(&args, &conf) {
                 print_iter(new_args, " ");
             } else {
-                print_iter(args.iter(), " ");
-            }
-            Ok(())
+                log::info!("並非別名");
+                return Err(Error::Completion);
+            };
         }
         Completion::Home { args } => {
             let root = parse_alias_root(&args)?;
             let home = root.root_args.hs_home.ok_or_else(|| Error::Completion)?;
             print!("{}", home);
-            Ok(())
+        }
+        Completion::ParseRun { args } => {
+            let mut root = Root::from_iter_safe(args).map_err(|e| {
+                log::warn!("補全時出錯 {}", e);
+                Error::Completion
+            })?;
+            root.sanitize()?;
+            match root.subcmd {
+                Some(Subs::Run {
+                    script_query, args, ..
+                }) => {
+                    let iter = std::iter::once(script_query.to_string())
+                        .chain(args.into_iter().map(|s| to_display_args(s)));
+                    print_iter(iter, " ");
+                }
+                res @ _ => {
+                    log::warn!("非執行指令 {:?}", res);
+                    return Err(Error::Completion);
+                }
+            }
         }
     }
+    Ok(())
 }
