@@ -1,7 +1,7 @@
 use crate::config::Config;
 use crate::error::{Contextable, Error, Result};
 use crate::extract_msg::extract_env_from_content;
-use crate::path::{get_home, get_template_path};
+use crate::path::{get_home, get_template_path, normalize_path};
 use crate::script::ScriptInfo;
 use crate::script_type::{get_default_template, ScriptType};
 use chrono::{DateTime, Utc};
@@ -13,14 +13,19 @@ use std::io::{Read, Write};
 use std::path::{Path, PathBuf};
 use std::process::{Command, ExitStatus};
 
+pub mod completion_util;
 pub mod main_util;
 pub mod serde;
 
+pub mod init_repo;
+pub use init_repo::init_repo;
+
 pub fn illegal_name(s: &str) -> bool {
     s.starts_with('-')
         || s.starts_with('.')
         || s.contains("..")
         || s.contains(' ')
+        || s.contains('!')
         || s.contains('/')
         || s.is_empty()
 }
@@ -279,7 +284,7 @@ pub fn prepare_script<T: AsRef<str>>(
     let has_content = !content.is_empty();
     let is_new = !path.exists();
     if is_new {
-        let birthplace_abs = handle_fs_res(&["."], std::env::current_dir())?;
+        let birthplace_abs = normalize_path(".")?;
         let birthplace = relative_to_home(&birthplace_abs);
 
         // NOTE: 創建資料夾和檔案
@@ -344,6 +349,7 @@ fn write_prepare_script<W: Write>(
         })
 }
 
+/// 可用來表示「未知類別」的概念 TODO: 測試之
 pub struct DisplayType<'a> {
     ty: &'a ScriptType,
     color: Option<Color>,
