@@ -219,7 +219,7 @@ fn test_rm() {
     assert!(!check_exist(&["longlive.sh"]));
     run!("-f all which").expect_err("沒有確實消滅掉一切");
 
-    // ---- 測試用 ! 來刪除 ----
+    // NOTE: ---- 測試用 ! 來刪除 ----
     run!("e -t hide hidden | echo 隱藏腳本，請用 ! 刪除我").unwrap();
     run!("rm --purge hidden").expect_err("沒加 ! 就刪掉了？");
     run!("rm --purge hidden!").expect("用了 ! 沒刪成功？");
@@ -242,34 +242,41 @@ fn test_namespace_reorder_search() {
 #[test]
 fn test_append_tags() {
     let _g = setup();
+    const CONTENT: Option<&str> = Some("echo $HS_TAGS");
     run!("tags global,test").unwrap();
-    let append_test = ScriptTest::new("append-test", Some("+append"));
-    let no_append_test = ScriptTest::new("no-append-test", Some("no-append"));
 
-    append_test.assert_tags(["global", "test", "append"], None, None);
+    pub fn assert_tags<const N: usize>(script: &ScriptTest, tags: [&str; N]) {
+        let res = script.filter("-a").run("").unwrap();
+        let mut actual_tags: Vec<_> = res.split(' ').filter(|s| !s.is_empty()).collect();
+        actual_tags.sort();
+        let mut expected_tags: Vec<_> = tags.iter().map(|s| *s).collect();
+        expected_tags.sort();
+        assert_eq!(expected_tags, actual_tags, "{:?} 的標籤不如預期", script);
+    }
+
+    let append_test = ScriptTest::new("append-test", Some("+append"), CONTENT);
+    let no_append_test = ScriptTest::new("no-append-test", Some("no-append"), CONTENT);
+
+    assert_tags(&append_test, ["global", "test", "append"]);
     no_append_test.assert_not_exist(None, None);
-    no_append_test.assert_tags(["no-append"], Some("-a"), None);
+    assert_tags(&no_append_test, ["no-append"]);
 
     run!("mv -f no-append no-append-test -t +eventually-append").unwrap();
 
-    no_append_test.assert_tags(
-        ["no-append", "eventually-append"],
-        Some("-a"),
-        Some("未能以 mv 附加標籤"),
-    );
-    append_test.assert_tags(["global", "test", "append"], None, Some("標籤被 mv 弄壞了"));
+    assert_tags(&no_append_test, ["no-append", "eventually-append"]);
+    assert_tags(&append_test, ["global", "test", "append"]);
     // 測試 ^all 的功能（無視一切先前的標籤）
-    let t1 = ScriptTest::new("non-global/namespace-test", Some("+^all"));
-    let t2 = ScriptTest::new("non-global/no-namespace-test", Some("^all"));
-    t1.assert_tags(["non-global"], Some("-a"), None);
-    t2.assert_tags([], Some("-a"), None);
+    let t1 = ScriptTest::new("non-global/namespace-test", Some("+^all"), CONTENT);
+    let t2 = ScriptTest::new("non-global/no-namespace-test", Some("^all"), CONTENT);
+    assert_tags(&t1, ["non-global"]);
+    assert_tags(&t2, []);
     // 測試 ^{some-tag} 的功能
-    let t1 = ScriptTest::new("only-test", Some("+^global"));
-    let t2 = ScriptTest::new("only-global", Some("+^test"));
-    let t3 = ScriptTest::new("nooooo", Some("^test"));
-    t1.assert_tags(["test"], None, None);
-    t2.assert_tags(["global"], None, None);
-    t3.assert_tags([], Some("-a"), None);
+    let t1 = ScriptTest::new("only-test", Some("+^global"), CONTENT);
+    let t2 = ScriptTest::new("only-global", Some("+^test"), CONTENT);
+    let t3 = ScriptTest::new("nooooo", Some("^test"), CONTENT);
+    assert_tags(&t1, ["test"]);
+    assert_tags(&t2, ["global"]);
+    assert_tags(&t3, []);
 }
 
 #[test]
@@ -321,28 +328,36 @@ fn test_redirect() {
 }
 
 #[test]
-fn test_mandatory_tags() {
+fn test_mandatory_filter() {
     let _g = setup();
-    run!("e prj1/t | echo prj1/src").unwrap();
-    run!("e prj2/t | echo prj2/src").unwrap();
-    run!("e prj1/src/t | echo prj1/src").unwrap();
-    run!("e prj2/src/t | echo prj2/src").unwrap();
+    let t1 = ScriptTest::new("prj1/t", None, None);
+    let t2 = ScriptTest::new("prj2/t", None, None);
+    let t3 = ScriptTest::new("prj1/src/t", None, None);
+    let t4 = ScriptTest::new("prj2/src/t", None, None);
+    let t5 = ScriptTest::new("hide/prj2/src/t", None, None);
 
-    assert_ls(
-        vec!["prj1/t", "prj1/src/t", "prj2/t", "prj2/src/t"],
-        None,
-        None,
-    );
+    fn assert_ls_names<'a, const N: usize>(
+        arr: [&'a ScriptTest; N],
+        filter: Option<&str>,
+        query: Option<&str>,
+    ) {
+        let v: Vec<_> = arr.iter().map(|s| s.get_name()).collect();
+        assert_ls(v, filter, query);
+    }
 
-    run!("tags prj1").unwrap();
-    assert_ls(vec!["prj1/t", "prj1/src/t"], None, None);
-    assert_ls(vec!["prj1/src/t", "prj2/src/t"], Some("src"), None);
-    assert_ls(
-        vec!["prj1/t", "prj2/src/t", "prj1/src/t"],
-        Some("+src"),
-        None,
-    );
-    assert_ls(vec!["prj1/src/t"], Some("+m/src"), None);
+    assert_ls_names([&t1, &t2, &t3, &t4], None, None);
+    assert_ls_names([], None, Some("-f all!"));
+    // NOTE: 順便測試在參數上帶多個篩選器
+    assert_ls_names([&t3, &t4], None, Some("-f +src!"));
+    assert_ls_names([&t3, &t4], None, Some("-f +src! -f +prj1")); // +prj1 非強制，不影響
+    assert_ls_names([&t3], None, Some("-f +src! -f +prj1!")); // +prj1! 為強制，會影響
+
+    run!("tags +prj1").unwrap();
+    assert_ls_names([&t1, &t3], None, None);
+    assert_ls_names([&t3, &t4, &t5], Some("src"), None);
+    assert_ls_names([&t1, &t3, &t4], Some("+src"), None);
+    assert_ls_names([&t3], Some("+src!"), None);
+    assert_ls_names([&t4, &t5], None, Some("-f all -f +src! -f +prj2!"));
 }
 
 #[test]
@@ -415,3 +430,32 @@ fn test_ls_query() {
         Some("showfuz wildcar* - =hide/exact! fzhid! !"),
     );
 }
+
+#[test]
+fn test_miss_event() {
+    let _g = setup();
+    let hidden = ScriptTest::new("1", Some("hide"), None);
+    let neglected = ScriptTest::new("3", None, None);
+    let normal = ScriptTest::new("2", None, None);
+    run!("history neglect {}", neglected.get_name()).unwrap();
+
+    let test_all = || {
+        hidden.can_find("!").unwrap_err();
+        hidden.can_find_by_name().unwrap_err();
+        hidden.can_find("!").expect("錯過事件無效？");
+
+        neglected.can_find("!").unwrap_err();
+        neglected.can_find_by_name().expect_err("neglect 無效？");
+        neglected.can_find("!").expect("錯過事件無效？");
+        neglected
+            .can_find_by_name()
+            .expect_err("錯過事件打破了時間篩選器？");
+
+        normal.can_find_by_name().unwrap();
+        normal.can_find("-").unwrap();
+        normal.can_find("!").expect_err("亂製造錯過事件？");
+    };
+
+    test_all();
+    test_all();
+}
